@@ -1,17 +1,16 @@
-import numpy as np
 from abc import ABC, abstractmethod
 import json
 from typing import Union, List
 
-import pandas as pd
 import sqlalchemy as sql
-from sqlalchemy import ForeignKey
+from sqlalchemy import ForeignKey, Integer, MetaData
 from sqlalchemy.ext.compiler import compiles
 
 from mipdb.database import DataBase, Connection
 from mipdb.database import METADATA_SCHEMA
 from mipdb.database import METADATA_TABLE
 from mipdb.dataelements import CommonDataElement
+from mipdb.exceptions import UserInputError
 from mipdb.schema import Schema
 
 
@@ -30,6 +29,7 @@ class SQLTYPES:
 
 
 STR2SQLTYPE = {"int": SQLTYPES.INTEGER, "text": SQLTYPES.STRING, "real": SQLTYPES.FLOAT}
+TEMPORARY_TABLE_NAME = "temp"
 
 
 class Table(ABC):
@@ -52,6 +52,12 @@ class Table(ABC):
     def insert_values(self, values, db: Union[DataBase, Connection]):
         db.insert_values_to_table(self._table, values)
 
+    def delete(self, db: Union[DataBase, Connection]):
+        db.delete_table_values(self._table)
+
+    def get_row_count(self, db):
+        return db.get_row_count(self.table.fullname)
+
     def drop(self, db: Union[DataBase, Connection]):
         db.drop_table(self._table)
 
@@ -161,13 +167,16 @@ class DatasetsTable(Table):
     def create(self, db: Union[DataBase, Connection]):
         db.create_table(self.table)
 
-    def get_datasets(self, db, data_model_id=None, columns=None):
+    def get_values(self, db, data_model_id=None, columns=None):
         if columns and not set(columns).issubset(self.table.columns.keys()):
             non_existing_columns = list(set(columns) - set(self.table.columns.keys()))
             raise ValueError(
                 f"The columns: {non_existing_columns} do not exist in the datasets schema"
             )
-        return db.get_datasets(data_model_id, columns)
+        datasets = db.get_values(data_model_id, columns)
+        if columns and len(columns) == 1:
+            return [attribute for attribute, *_ in datasets]
+        return db.get_values(data_model_id, columns)
 
     def get_dataset(self, db, dataset_id=None, columns=None):
         if columns and not set(columns).issubset(self.table.columns.keys()):
@@ -264,7 +273,6 @@ class PrimaryDataTable(Table):
                 "row_id",
                 SQLTYPES.INTEGER,
                 primary_key=True,
-                autoincrement=True,
                 quote=True,
             ),
         )
@@ -336,3 +344,103 @@ class MetadataTable(Table):
 
     def get_dataset_enums(self):
         return json.loads(self.table["dataset"].metadata)["enumerations"]
+
+    def get_sql_type_per_column(self):
+        return {
+            code: json.loads(cde.metadata)["sql_type"]
+            for code, cde in self.table.items()
+        }
+
+    def get_cdes_with_min_max(self, columns):
+        cdes_with_min_max = {}
+        for code, cde in self.table.items():
+            if code not in columns:
+                continue
+            metadata = json.loads(cde.metadata)
+            max_value = metadata["max"] if "max" in metadata else None
+            min_value = metadata["min"] if "min" in metadata else None
+            if code in columns and min_value or max_value:
+                cdes_with_min_max[code] = (min_value, max_value)
+        return cdes_with_min_max
+
+    def get_cdes_with_enumerations(self, columns):
+        return {
+            code: [
+                enum_code
+                for enum_code, enum_label in json.loads(cde.metadata)[
+                    "enumerations"
+                ].items()
+            ]
+            for code, cde in self.table.items()
+            if json.loads(cde.metadata)["is_categorical"] and code in columns
+        }
+
+
+class TemporaryTable(Table):
+    def __init__(self, dataframe_sql_type_per_column, db):
+        columns = [
+            sql.Column(name, STR2SQLTYPE[sql_type], quote=True)
+            for name, sql_type in dataframe_sql_type_per_column.items()
+        ]
+
+        self._table = sql.Table(
+            TEMPORARY_TABLE_NAME,
+            MetaData(bind=db.get_executor()),
+            *columns,
+            prefixes=["TEMPORARY"],
+        )
+
+    def validate_data(self, cdes_with_min_max, cdes_with_enumerations, db):
+
+        self._validate_enumerations_restriction(cdes_with_enumerations, db)
+        self._validate_min_max_restriction(cdes_with_min_max, db)
+
+    def _validate_min_max_restriction(self, cdes_with_min_max, db):
+        for cde, min_max in cdes_with_min_max.items():
+            min_value, max_value = min_max
+            cde_invalid_values = db.execute(
+                f"SELECT \"{cde}\" FROM {self.table.fullname} WHERE \"{cde}\" NOT BETWEEN '{min_value}' AND '{max_value}' "
+            ).fetchone()
+            if cde_invalid_values:
+                raise Exception(
+                    f"In the column: '{cde}' the following values are invalid: '{cde_invalid_values}'"
+                )
+
+    def load_csv(
+        self,
+        csv_path,
+        db,
+        records=None,
+        offset=2,
+    ):
+        self._validate_csv_contains_eof(csv_path=csv_path)
+        db.copy_csv_in_table(
+            file_location=csv_path,
+            records=records,
+            offset=offset,
+            table_name=self.table.name,
+        )
+
+    def _validate_csv_contains_eof(self, csv_path):
+        with open(csv_path, "rb") as f:
+            last_line = f.readlines()[-1]
+        if not last_line.endswith(b"\n"):
+            raise UserInputError(f"CSV:'{csv_path}' does not end with a blank line.")
+
+    def _validate_enumerations_restriction(self, cdes_with_enumerations, db):
+        for cde, enumerations in cdes_with_enumerations.items():
+            cde_invalid_values = db.execute(
+                f'SELECT "{cde}" from {self.table.fullname} where "{cde}" not in ({str(enumerations)[1:-1]})'
+            ).fetchone()
+            if cde_invalid_values:
+                raise Exception(
+                    f"In the column: '{cde}' the following values are invalid: '{cde_invalid_values}'"
+                )
+
+    def get_unique_datasets(self, db):
+        return db.execute(
+            f"SELECT DISTINCT(dataset) FROM {self.table.fullname};"
+        ).fetchone()
+
+    def set_table(self, table):
+        self._table = table
