@@ -7,7 +7,8 @@ import pandas as pd
 
 from mipdb.database import DataBase, Connection
 from mipdb.database import METADATA_SCHEMA
-from mipdb.exceptions import ForeignKeyError
+from mipdb.data_frame_schema import DataFrameSchema
+from mipdb.exceptions import ForeignKeyError, InvalidDatasetError
 from mipdb.exceptions import UserInputError
 from mipdb.properties import Properties
 from mipdb.reader import CSVDataFrameReader
@@ -22,8 +23,12 @@ from mipdb.tables import (
     ActionsTable,
     MetadataTable,
     PrimaryDataTable,
+    TemporaryTable,
 )
-from mipdb.dataset import Dataset
+from mipdb.data_frame import DataFrame
+
+DATASET_COLUMN_NAME = "dataset"
+RECORDS_PER_COPY = 100000
 
 
 class UseCase(ABC):
@@ -167,7 +172,7 @@ class DeleteDataModel(UseCase):
     def _validate_data_model_deletion(self, data_model_name, data_model_id, conn):
         metadata = Schema(METADATA_SCHEMA)
         datasets_table = DatasetsTable(schema=metadata)
-        datasets = datasets_table.get_datasets(conn, data_model_id)
+        datasets = datasets_table.get_values(conn, data_model_id)
         if not len(datasets) == 0:
             raise ForeignKeyError(
                 f"The Data Model:{data_model_name} cannot be deleted because it contains Datasets: {datasets}"
@@ -178,10 +183,9 @@ class DeleteDataModel(UseCase):
         metadata = Schema(METADATA_SCHEMA)
         datasets_table = DatasetsTable(schema=metadata)
         with self.db.begin() as conn:
-            dataset_rows = datasets_table.get_datasets(
+            dataset_codes = datasets_table.get_values(
                 data_model_id=data_model_id, columns=["code"], db=conn
             )
-        dataset_codes = [dataset_row[0] for dataset_row in dataset_rows]
 
         for dataset_code in dataset_codes:
             DeleteDataset(self.db).execute(
@@ -191,12 +195,14 @@ class DeleteDataModel(UseCase):
             )
 
 
-class AddDataset(UseCase):
+class ImportCSV(UseCase):
     def __init__(self, db: DataBase) -> None:
         self.db = db
         is_db_initialized(db)
 
-    def execute(self, csv_path, data_model_code, data_model_version) -> None:
+    def execute(
+        self, csv_path, copy_from_file, data_model_code, data_model_version
+    ) -> None:
         data_model_name = get_data_model_fullname(
             code=data_model_code, version=data_model_version
         )
@@ -206,48 +212,44 @@ class AddDataset(UseCase):
         datasets_table = DatasetsTable(schema=metadata)
 
         with self.db.begin() as conn:
-            metadata_table = MetadataTable.from_db(data_model, conn)
-            dataset_enumerations = metadata_table.get_dataset_enums()
-            dataset_id = self._get_next_dataset_id(conn)
             data_model_id = data_model_table.get_data_model_id(
                 data_model_code, data_model_version, conn
             )
+            metadata_table = MetadataTable.from_db(data_model, conn)
+            dataset_enumerations = metadata_table.get_dataset_enums()
+            sql_type_per_column = metadata_table.get_sql_type_per_column()
 
-            primary_data_table = PrimaryDataTable.from_db(data_model, conn)
-            with CSVDataFrameReader(csv_path).get_reader() as reader:
-                for dataset_data in reader:
-                    dataset = Dataset(dataset_data)
+            if copy_from_file:
+                imported_datasets = self.import_csv_with_volume(
+                    csv_path=csv_path,
+                    sql_type_per_column=sql_type_per_column,
+                    data_model=data_model,
+                    conn=conn,
+                )
+            else:
+                imported_datasets = self._import_csv(
+                    csv_path=csv_path, data_model=data_model, conn=conn
+                )
 
-                    dataset.validate_dataset(metadata_table.table)
-            self._verify_dataset_does_not_exist(
-                data_model_id=data_model_id,
-                dataset_name=dataset.name,
-                conn=conn,
-            )
-            with CSVDataFrameReader(csv_path).get_reader() as reader:
-                for dataset_data in reader:
-                    dataset = Dataset(dataset_data)
-                    values = dataset.to_dict()
-                    primary_data_table.insert_values(values, conn)
-            label = dataset_enumerations[dataset.name]
-
-            values = dict(
-                data_model_id=data_model_id,
-                dataset_id=dataset_id,
-                code=dataset.name,
-                label=label,
-                status="ENABLED",
-            )
-            datasets_table.insert_values(values, conn)
-
-            data_model_details = _get_data_model_details(data_model_id, conn)
-            dataset_details = _get_dataset_details(dataset_id, conn)
-            update_actions(
-                conn=conn,
-                action="ADD DATASET",
-                data_model_details=data_model_details,
-                dataset_details=dataset_details,
-            )
+            existing_datasets = datasets_table.get_values(columns=["code"], db=conn)
+            for dataset in set(imported_datasets) - set(existing_datasets):
+                dataset_id = self._get_next_dataset_id(conn)
+                values = dict(
+                    data_model_id=data_model_id,
+                    dataset_id=dataset_id,
+                    code=dataset,
+                    label=dataset_enumerations[dataset],
+                    status="ENABLED",
+                )
+                datasets_table.insert_values(values, conn)
+                data_model_details = _get_data_model_details(data_model_id, conn)
+                dataset_details = _get_dataset_details(dataset_id, conn)
+                update_actions(
+                    conn=conn,
+                    action="ADD DATASET",
+                    data_model_details=data_model_details,
+                    dataset_details=dataset_details,
+                )
 
     def _get_next_dataset_id(self, conn):
         metadata = Schema(METADATA_SCHEMA)
@@ -255,15 +257,84 @@ class AddDataset(UseCase):
         dataset_id = datasets_table.get_next_dataset_id(conn)
         return dataset_id
 
-    def _verify_dataset_does_not_exist(self, data_model_id, dataset_name, conn):
-        metadata = Schema(METADATA_SCHEMA)
-        dataset_table = DatasetsTable(schema=metadata)
-        datasets = dataset_table.get_datasets(
-            db=conn, data_model_id=data_model_id, columns=["code"]
-        )
+    def _create_temporary_table(self, dataframe_sql_type_per_column, conn):
+        temporary_table = TemporaryTable(dataframe_sql_type_per_column, conn)
+        temporary_table.create(conn)
+        return temporary_table
 
-        if datasets and (dataset_name,) in datasets:
-            raise UserInputError("Dataset already exists!")
+    def import_csv_with_volume(self, csv_path, sql_type_per_column, data_model, conn):
+        csv_columns = pd.read_csv(csv_path, nrows=0).columns.tolist()
+        dataframe_sql_type_per_column = {
+            dataframe_column: sql_type_per_column[dataframe_column]
+            for dataframe_column in csv_columns
+        }
+        temporary_table = self._create_temporary_table(
+            dataframe_sql_type_per_column, conn
+        )
+        imported_datasets = self.insert_csv_to_db(
+            csv_path, temporary_table, data_model, conn
+        )
+        temporary_table.drop(conn)
+        return imported_datasets
+
+    # The monetdb copy into prohibites the importion of a csv,
+    # to a table if the csv contains fewer columns than the table.
+    # In our case we need to load the csv a table named 'primary_data'.
+    # Minimal example:
+    # TABLE 'primary_data': col1 col2 | CSV: col1
+    #                       1    2    |      5
+    #                       2    4    |      6
+    #                       3    6    |      7
+    #                       4    8    |      8
+    # We need to create a 'temp' table which will mirror the columns of the csv.
+    # Thus making the copy into a valid choice again.
+    # The workaround for that is that we first copy the value into the 'temp'.
+    # Once the data is loaded in the 'temp',
+    # we can now pass the values of the 'temp' to the 'primary_data' table using a query with the format:
+    # INSERT INTO table1 (columns)
+    # SELECT columns
+    # FROM table2;
+    # What about the missing columns at that the 'primary_data' contains but the 'temp' does not contain,
+    # we will simply add null in place of the missing columns.
+    # In our case the query will have the form:
+    # INSERT INTO 'primary_data' (col1, col2)
+    # SELECT col1, NULL
+    # FROM 'temp'
+
+    def insert_csv_to_db(self, csv_path, temporary_table, data_model, db):
+        primary_data_table = PrimaryDataTable.from_db(data_model, db)
+        offset = 2
+        imported_datasets = []
+        # If we load a csv to 'temp' and then insert them to the 'primary_data' in the case of a big file (3gb),
+        # for a sort period of time will have a spike of memory usage because the data will be stored in both tables.
+        # The workaround for that is to load the csv in batches.
+        while True:
+            temporary_table.load_csv(csv_path=csv_path, records=RECORDS_PER_COPY, db=db)
+            imported_datasets = set(imported_datasets) | set(
+                temporary_table.get_unique_datasets(db)
+            )
+            db.copy_data_table_to_another_table(primary_data_table, temporary_table)
+            temporary_table.delete(db)
+
+            # If the temp contains fewer rows than RECORDS_PER_COPY
+            # that means we have read all the records in the csv and we need to stop the iteration.
+            table_count = temporary_table.get_row_count(db=db)
+            if table_count < RECORDS_PER_COPY:
+                break
+
+            offset += RECORDS_PER_COPY
+        return imported_datasets
+
+    def _import_csv(self, csv_path, data_model, conn):
+        imported_datasets = []
+        primary_data_table = PrimaryDataTable.from_db(data_model, conn)
+        with CSVDataFrameReader(csv_path).get_reader() as reader:
+            for dataset_data in reader:
+                dataframe = DataFrame(dataset_data)
+                imported_datasets = set(imported_datasets) | set(dataframe.datasets)
+                values = dataframe.to_dict()
+                primary_data_table.insert_values(values, conn)
+        return imported_datasets
 
 
 class ValidateDataset(UseCase):
@@ -271,21 +342,98 @@ class ValidateDataset(UseCase):
         self.db = db
         is_db_initialized(db)
 
-    def execute(self, csv_path, data_model_code, data_model_version) -> None:
+    def execute(
+        self, csv_path, copy_from_file, data_model_code, data_model_version
+    ) -> None:
         data_model_name = get_data_model_fullname(
             code=data_model_code, version=data_model_version
         )
         data_model = Schema(data_model_name)
 
         with self.db.begin() as conn:
-            self.validate_csv(csv_path, data_model, conn)
+            csv_columns = pd.read_csv(csv_path, nrows=0).columns.tolist()
+            if DATASET_COLUMN_NAME not in csv_columns:
+                raise InvalidDatasetError(
+                    "The 'dataset' column is required to exist in the csv."
+                )
+            metadata_table = MetadataTable.from_db(data_model, conn)
+            sql_type_per_column = metadata_table.get_sql_type_per_column()
+            cdes_with_min_max = metadata_table.get_cdes_with_min_max(csv_columns)
+            cdes_with_enumerations = metadata_table.get_cdes_with_enumerations(
+                csv_columns
+            )
+            dataset_enumerations = metadata_table.get_dataset_enums()
+            if copy_from_file:
+                validated_datasets = self.validate_csv_with_volume(
+                    csv_path,
+                    sql_type_per_column,
+                    cdes_with_min_max,
+                    cdes_with_enumerations,
+                    conn,
+                )
+            else:
+                validated_datasets = self.validate_csv(
+                    csv_path,
+                    sql_type_per_column,
+                    cdes_with_min_max,
+                    cdes_with_enumerations,
+                )
+            self.verify_datasets_exist_in_enumerations(
+                datasets=validated_datasets,
+                dataset_enumerations=dataset_enumerations,
+            )
 
-    def validate_csv(self, csv_path, data_model, conn):
-        metadata_table = MetadataTable.from_db(data_model, conn)
+    def validate_csv(
+        self, csv_path, sql_type_per_column, cdes_with_min_max, cdes_with_enumerations
+    ):
+        imported_datasets = []
+
+        csv_columns = pd.read_csv(csv_path, nrows=0).columns.tolist()
+        dataframe_schema = DataFrameSchema(
+            sql_type_per_column, cdes_with_min_max, cdes_with_enumerations, csv_columns
+        )
         with CSVDataFrameReader(csv_path).get_reader() as reader:
             for dataset_data in reader:
-                dataset = Dataset(dataset_data)
-                dataset.validate_dataset(metadata_table.table)
+                dataframe = DataFrame(dataset_data)
+                dataframe_schema.validate_dataframe(dataframe.data)
+                imported_datasets = set(imported_datasets) | set(dataframe.datasets)
+        return imported_datasets
+
+    def validate_csv_with_volume(
+        self,
+        csv_path,
+        sql_type_per_column,
+        cdes_with_min_max,
+        cdes_with_enumerations,
+        conn,
+    ):
+        csv_columns = pd.read_csv(csv_path, nrows=0).columns.tolist()
+        dataframe_sql_type_per_column = {
+            dataframe_column: sql_type_per_column[dataframe_column]
+            for dataframe_column in csv_columns
+        }
+        temporary_table = self._create_temporary_table(
+            dataframe_sql_type_per_column, conn
+        )
+        temporary_table.load_csv(csv_path=csv_path, db=conn)
+        validated_datasets = temporary_table.get_unique_datasets(conn)
+        temporary_table.validate_data(cdes_with_min_max, cdes_with_enumerations, conn)
+        temporary_table.drop(conn)
+        return validated_datasets
+
+    def _create_temporary_table(self, dataframe_sql_type_per_column, conn):
+        temporary_table = TemporaryTable(dataframe_sql_type_per_column, conn)
+        temporary_table.create(conn)
+        return temporary_table
+
+    def verify_datasets_exist_in_enumerations(self, datasets, dataset_enumerations):
+        non_existing_datasets = [
+            dataset for dataset in datasets if dataset not in dataset_enumerations
+        ]
+        if non_existing_datasets:
+            raise InvalidDatasetError(
+                f"The values:'{non_existing_datasets}' are not present in the enumerations of the CDE 'dataset'."
+            )
 
 
 class DeleteDataset(UseCase):
@@ -747,7 +895,7 @@ class ListDatasets(UseCase):
                 "label",
                 "status",
             ]
-            dataset_rows = dataset_table.get_datasets(conn, columns=dataset_row_columns)
+            dataset_rows = dataset_table.get_values(conn, columns=dataset_row_columns)
 
             data_model_fullname_by_data_model_id = {
                 data_model_id: get_data_model_fullname(code, version)
